﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Claims;
using System.Web;

namespace APISampleWebSite
{
	public static class ClaimsIdentityExtensions
	{
		/// <summary>
		/// Get Claim Value from collection
		/// </summary>
		/// <param name="claims"></param>
		/// <param name="claimType"></param>
		/// <returns></returns>
		public static string GetClaimValue(this ClaimsIdentity identity, string claimType)
		{
			string value = string.Empty;

			try
			{
				Claim claim = identity.GetClaim(claimType);

				if (claim != null)
				{
					value = claim.Value;
				}
			}
			catch
			{
				// just return empty string
			}

			return value;
		}

		/// <summary>
		/// Get Claim from collection
		/// </summary>
		/// <param name="claims"></param>
		/// <param name="claimType"></param>
		/// <returns></returns>
		public static Claim GetClaim(this ClaimsIdentity identity, string claimType)
		{
			Claim claim = null;

			try
			{
				claim = identity.Claims
						.Where(c => c.Type == claimType)
						.FirstOrDefault();

			}
			catch
			{
				// just return null;
			}


			return claim;
		}

		/// <summary>
		/// Set new value to claim
		/// </summary>
		/// <param name="claims"></param>
		/// <param name="claimType"></param>
		/// <param name="value"></param>
		/// <remarks>
		/// Removes any existing claim, and replaces by brand new one.
		/// </remarks>
		public static void SetClaim(this ClaimsIdentity identity, string claimType, string value)
		{
			Claim newClaim = new Claim(claimType, value);

			identity.SetClaim(newClaim);
		}

		/// <summary>
		/// Inserts or replaces claim
		/// </summary>
		/// <param name="identity"></param>
		/// <param name="newClaim"></param>
		public static void SetClaim(this ClaimsIdentity identity, Claim newClaim)
		{
			Claim oldClaim = identity.GetClaim(newClaim.Type);

			if (oldClaim != null)
			{
				identity.TryRemoveClaim(oldClaim);
			}

			identity.AddClaim(newClaim);
		}

	}
}